#!/usr/bin/python
import urwid
import json
import sys
import logging
import os
import subprocess
import traceback
from list_button import list_button

# Program settings UI
class terminal_overflow_settings_popup(urwid.WidgetWrap):
    signals = ['close']
    
    checkbox_category_group = []
    text_checkbox_list = [u"history", u"social", u"saved"]
    checkboxes = []
    selected_category = None


    #category selected
    def checkbox_item_chosen(self, button, newstate, selected_category):
        if newstate is True or newstate is "firstTrue":
            self.selected_category=selected_category

    def generate_checkbox_menu(self, choices):
        self.checkboxes = []
        for checkbox_category in choices:
            checkbox_item = urwid.CheckBox(checkbox_category,state=self.settings_manager.get_category_enabled(checkbox_category))
            urwid.connect_signal(checkbox_item, 'change', self.checkbox_item_chosen, checkbox_category)
            checkbox_item=urwid.AttrWrap(checkbox_item, 'radregular','radfocus')           
            self.checkboxes.append(checkbox_item)
            columns_checkboxes = urwid.Columns(self.checkboxes,dividechars=1, focus_column=0)
        return columns_checkboxes
    
    def save_settings_click(self):

        settings_urls = {"history":self.history_file_widget.get_edit_text(),
                         "social":self.social_url_widget.get_edit_text(),
                         "saved":self.saved_url_widget.get_edit_text()}

        categories_enabled = {}

        for current_checkbox in self.checkboxes:
            categories_enabled[current_checkbox.get_label()] = current_checkbox.get_state()

        settings_text={"settings_urls":settings_urls,"categories_enabled":categories_enabled}
        self.settings_manager.write_user_settings(settings_text)
        self.settings_manager.load_user_settings()
        self._emit("close")
        

    def __init__(self,palette,settings_manager):

        # Get current instance of settings manager
        self.settings_manager = settings_manager

        # Settings headers
        self.settings_category_header = urwid.Text(u'Enabled Categories')
        self.settings_social_header = urwid.Text(u'Social URL')
        self.settings_saved_header = urwid.Text(u'Saved URL')
        self.settings_history_header = urwid.Text(u'History File')

        # Save and Close Buttons
        self.save_button_widget = list_button("[ Update ]")
        self.close_button_widget = list_button("[ Cancel ]")
        self.save_button = urwid.AttrWrap((self.save_button_widget),'settings_edit_button_save','settings_edit_button_focus')
        self.close_button = urwid.AttrWrap((self.close_button_widget),'settings_edit_button_close','settings_edit_button_focus')

        urwid.connect_signal(self.save_button_widget, 'click',lambda button:self.save_settings_click())
        urwid.connect_signal(self.close_button_widget, 'click',lambda button:self._emit("close"))
        
        # Edit Saved URL
        self.saved_url_widget = urwid.Edit(u"",self.settings_manager.get_user_settings("saved"))
        self.saved_url_widget.set_caption(('editbx','Search: '))
        self.saved_url = urwid.AttrWrap((self.saved_url_widget),'settings_edit_text','settings_edit_text_focus')  
        # Edit History File
        self.history_file_widget = urwid.Edit(u"",self.settings_manager.get_user_settings("history"))
        self.history_file_widget.set_caption(('editbx','File: '))
        self.history_file = urwid.AttrWrap((self.history_file_widget),'settings_edit_text','settings_edit_text_focus') 
        # Edit Social URL
        self.social_url_widget = urwid.Edit(u"",self.settings_manager.get_user_settings("social"))
        self.social_url_widget.set_caption(('editbx','File: '))
        self.social_url = urwid.AttrWrap((self.social_url_widget),'settings_edit_text','settings_edit_text_focus')

        # Generate checkbox button categories
        self.checkbox_category_menu = self.generate_checkbox_menu(self.text_checkbox_list)


        self.button_cols = urwid.GridFlow([self.save_button,self.close_button],10,5,0,align="center")

        self.pile_settings = urwid.Pile([
                                    self.settings_category_header,
                                    self.checkbox_category_menu,
                                    self.settings_social_header,
                                    self.social_url,
                                    self.settings_saved_header,
                                    self.saved_url,
                                    self.settings_history_header,
                                    self.history_file,
                                    self.button_cols
                                    ])

                                    
        settings_filler = urwid.Filler(self.pile_settings, valign='top', top=0, bottom=1)
        settings_box_details = urwid.LineBox(settings_filler)
        
        super(terminal_overflow_settings_popup, self).__init__(urwid.AttrWrap(settings_box_details, ''))


# Settings popup widget
class terminal_overflow_settings_module(urwid.PopUpLauncher):
    import list_button

    isOpen = False
    palette = []

    def __init__(self,title,palette,settings_manager):
        
        self.palette = palette
        self.settings_manager = settings_manager

        the_popup_button = list_button(title)
        super(terminal_overflow_settings_module, self).__init__(the_popup_button)
        urwid.connect_signal(self.original_widget, 'click',lambda button: self.toggle_open_close())
        
    def create_pop_up(self):
        pop_up = terminal_overflow_settings_popup(self.palette,self.settings_manager)
        urwid.connect_signal(pop_up, 'close',lambda button: self.toggle_open_close())
        return pop_up

    def get_pop_up_parameters(self):
        return {'left':0, 'top':1, 'overlay_width':75, 'overlay_height':14}                

    def toggle_open_close(self):
        if self.isOpen:
            self.isOpen = False
            self.close_pop_up()
            
        else:
            self.isOpen = True
            self.open_pop_up()



import urwid.wimp
import urwid

from urwid.wimp import *

class CustomButton(WidgetWrap):
    def sizing(self):
        return frozenset([FLOW])
    button_left = Text("<")
    button_right = Text(">")
    signals = ["click"]

    def __init__(self, label, on_press=None, user_data=None):       
        self._label = self._label = ButtonLabel("")
        cols = Columns([self._label],dividechars=0)
        super(CustomButton, self).__init__(cols)
        if on_press:
            connect_signal(self, 'click', on_press, user_data)

        self.set_label(label)

    def _repr_words(self):
        return super(CustomButton, self)._repr_words() + [
            python3_repr(self.label)]
    def set_label(self, label):
        
        self._label.set_text(label)

    def get_label(self):
        
        return self._label.text
    label = property(get_label)

    def keypress(self, size, key):
        
        if self._command_map[key] != ACTIVATE:
            return key

        self._emit('click')

    def mouse_event(self, size, event, button, x, y, focus):
        
        if button != 1 or not is_mouse_press(event):
            return False

        self._emit('click')
        return True        
          
class ButtonLabel(urwid.SelectableIcon):
    def set_text(self, label):
        super(ButtonLabel, self).set_text(label)
        self._cursor_position = len(label) + 1

class list_button(CustomButton):
    pass



#!/usr/bin/python
import urwid
from urllib2 import urlopen
from HTMLParser import HTMLParser
from simplejson import loads

from history_command_manager import history_command_manager
from settings_manager import settings_manager
from social_search_manager import social_search_manager
from list_button import list_button
from command_parse_manager import command_parse_manager

import json
import sys
import logging
import os
import subprocess
import traceback

class terminal_overflow_search_module():

################### SETTINGS ################### 

    initial_search_results = ["none"]
    current_screen = urwid.raw_display.Screen()
    screen_cols,screen_rows = current_screen.get_cols_rows()
    current_list_height = screen_rows - 6
    current_list_width = (screen_cols/2) -10
    current_command_token_list_width = (screen_cols/2) -5
    radio_category_group = []
    text_radio_list = [u"history", u"social", u"saved"]
    radios = []
    selected_category="history"
 
    initial_search_result_details = {"none":["none"]}
    palette = [ ]
    
    
    def generate_radio_menu(self, choices):
        for rad_choice in choices:
            radio_button = urwid.RadioButton(self.radio_category_group, rad_choice)
            urwid.connect_signal(radio_button, 'change', self.radio_item_chosen, rad_choice)
            radio_button=urwid.AttrWrap(radio_button, 'radregular','radfocus')           
            self.radios.append(radio_button)
            columns_radios = urwid.Columns(self.radios,dividechars=1, focus_column=0)
        return columns_radios

    def update_radio_category_count(self,category_name,category_count):     
        for current_radio in self.radios:
            if category_name in current_radio.get_label():
                current_radio.set_label(category_name + "["+ str(category_count) + "]")

    # switch to and display results in selected category
    def load_search_category(self,pressed_enter=False,search_box_text=None):
            if search_box_text is not None:
                search_text = search_box_text
            else:
                search_text = self.ask.get_edit_text()
            search_results = []

            if "social" in self.selected_category:
                if pressed_enter:
                    tmp_reddit_data = self.social_search_manager.run_search(self.settings_manager.get_user_settings('social'))
                    search_results = list(filter(lambda x: search_text in x, tmp_reddit_data))
                    search_results = [self.format_search_result(x,self.selected_category) for x in search_results]
                    self.main_results.original_widget = self.generate_menu(u'', search_results)
                    self.update_radio_category_count(self.selected_category,len(search_results))
                else:
                    pass
                    #on social search dont update for every key press
                    #wait till user presses enter, todo: put timer here?
            elif "history" in self.selected_category:
                tmp_history_data = self.history_command_manager.get_history_list(self.settings_manager.get_user_settings("history"))
                search_results = list(filter(lambda x: search_text in x, tmp_history_data))
                search_results = [self.format_search_result(x,self.selected_category) for x in search_results]
                self.main_results.original_widget = self.generate_menu(u'', search_results)
                self.update_radio_category_count(self.selected_category,len(search_results))     
            elif "saved" in self.selected_category:
                pass
            else:
                pass

    
################### FORMAT ###################   

    #Search Results
    def format_search_result(self,unformatted_result,search_results_type):
        max_line_width = self.current_list_width    
        formatted_result = ""
        if "social" in search_results_type:
            formatted_result = " social " + unformatted_result[:max_line_width]
        elif "history" in search_results_type:
            tmp_string_list = unformatted_result.split(" ")
            tmp_string = "[#"+tmp_string_list[1]+"] " + " ".join(tmp_string_list[4:])
            formatted_result = tmp_string[:max_line_width]
        elif "saved" in search_results_type:
            formatted_result = " saved " + unformatted_result[:max_line_width]
        else:
            formatted_result = " bad search result type"
        return formatted_result

################### GENERATE ###################

    #Search Categories
    def generate_menu(self, title, choices):
        body = []
        for c in choices:
            button = list_button(c)
            urwid.connect_signal(button, 'click', self.item_chosen, c)
            #button = urwid.Padding(button,align='left',width='clip')
            body.append(urwid.AttrMap(button,None, focus_map='reversed'))       
        return urwid.ListBox(urwid.SimpleFocusListWalker(body))


################### CLICK EVENTS ###################
    
    #Category
    def radio_item_chosen(self, button, newstate, selected_category):
        if newstate is True or newstate is "firstTrue":
            self.selected_category = selected_category
            self.load_search_category(True)

    #Search
    def item_chosen(self,button, choice):
        self.publish_subscribe_manager.publish_event('search_input',choice)
        pass
        

################### KEY EVENTS ###################

    def on_search_input_change(self,edit, new_edit_text):
        self.load_search_category(False,new_edit_text)
        pass

    # Handles Unhandled MainLoop keyboard input
    def handle_input(self,key):

        # Module specific key
        if key == 'ctrl x':
            raise urwid.ExitMainLoop()
        

    def __init__(self):


################### CREATE MANAGERS ###################
        #Settings Manager
        self.settings_manager = settings_manager()
        self.history_command_manager = history_command_manager()
        self.social_search_manager = social_search_manager()

################### CREATE UI ###################

       
    def build_initial_layout(self):

         # Text input section
        self.ask_widget = urwid.Edit(u"")
        self.ask_widget.set_caption(('editbx','Search: '))
        self.ask = urwid.AttrWrap((self.ask_widget),'editbx','editfc')  
        urwid.connect_signal(self.ask.original_widget, 'change', self.on_search_input_change)

        # Generate radio button categories
        self.radio_category_menu = self.generate_radio_menu(self.text_radio_list)

        # Generate initial search result list area
        self.main_results = urwid.BoxAdapter(self.generate_menu(u'', self.initial_search_results),self.current_list_height)
        self.pile = urwid.Pile([self.ask,self.radio_category_menu,  self.main_results])
        self.quote_filler = urwid.Filler(self.pile, valign='top', top=0, bottom=1)
        self.quote_box = urwid.LineBox(self.quote_filler)

        # Assemble the widgets into the widget layout
        self.layout = urwid.Frame(header=None, body=self.quote_box, footer=None)

        
    def get_base_layout_widget(self):
        return self.layout  

    def begin(self,publish_subscribe_manager):
        self.publish_subscribe_manager = publish_subscribe_manager
        
        #load initial search results "everything"
        self.load_search_category()


logging.debug('Loading Search Module')




import json
import sys
import logging
import os
import string
import traceback


class settings_manager():

    default_settings_urls = {"history":"~/.bash_history",
                            "social":"https://www.reddit.com/r/frontpage/.json",
                            "saved":"https://www.github.com"}

    default_categories_enabled = {"history":True,
                                  "social":False,
                                  "saved":True}

    config_file_default_location="~/terminal_overflow.config"

    current_settings_urls = {}
    current_settings_categories_enabled = {}
    

    def __init__(self):
        self.load_user_settings()
        pass

    # Gets the settings url for a category    
    def get_user_settings(self,category):
        return self.current_settings_urls[category]
    # Gets category enabled setting
    def get_category_enabled(self,category):
        return self.current_settings_categories_enabled[category]
    # Gets the application config file location
    def get_config_location(self):
        return self.config_file_default_location

    def has_printable_chars(self,test_string):
        printable_set = set(string.printable)
        tmpset = set(json.dumps(test_string))
        tmpset.difference_update(printable_set)
       
        if len(tmpset) is not 0:
            #writing the invalid chars can mess up logfile
            #logging.error("ERROR: Invalid characters in settings file:"+str(tmpset))
            return False
        else:
            return True
        
    
    def parse_user_settings(self,settings_data):

        if not self.has_printable_chars(settings_data):
            logging.error("Invalid characters in settings file")
            return False

        try:
            #TODO: check if settings urls are properly formatted
            parsed_settings_urls = settings_data["settings_urls"]
            parsed_categories_enabled = settings_data["categories_enabled"]

            for tmp_category in parsed_categories_enabled:
                tmp_str = str(parsed_categories_enabled[tmp_category])
                if tmp_str.upper() == "TRUE":
                    parsed_categories_enabled[tmp_category]=True
                else:
                    parsed_categories_enabled[tmp_category]=False
            
        except:
            logging.error("Invalid settings detected")
            return False
        

        self.current_settings_urls = parsed_settings_urls
        self.current_settings_categories_enabled = parsed_categories_enabled

        return True


    # Write the settings to the config file
    def write_user_settings(self,settings_text):
        
        write_success = True
        try:
            settings_location = os.path.expanduser(self.config_file_default_location)
            with open(settings_location, 'w') as f:
                json.dump(settings_text, f)
        except:
            write_success = False
            logging.error(traceback.format_exc())


        if write_success:
            logging.debug("Wrote settings file: " +settings_location)         
        else:
            logging.debug("Failed writing settings files: "+settings_location)

        #logging.debug("SETTINGS:"+str(settings_text))

    # Read the settings to the config file
    def load_user_settings(self):

        settings_location = os.path.expanduser(self.config_file_default_location)

        logging.debug("Loading settings: "+ settings_location)
        use_default_settings = False
        try:
            with open(settings_location, 'r') as f:
                file_content = json.load(f)

            if not file_content:
                use_default_settings = True
                logging.debug("Settings file is empty")
            else:
                if not self.parse_user_settings(file_content):
                    use_default_settings = True
                    logging.error("failed settings check")
        except:
            logging.error(traceback.format_exc())
            use_default_settings = True


        if use_default_settings:
            self.current_settings_urls = dict(self.default_settings_urls)
            self.current_settings_categories_enabled = dict(self.default_categories_enabled)
            logging.debug("Using default settings")
        
        logging.debug("Loaded settings"+str(self.current_settings_urls))
        logging.debug("Loaded settings:"+str(self.current_settings_categories_enabled))

        return   



#!/usr/bin/python

class publish_subscribe_manager():

    listeners={}

    def __init__(self):
        pass

    #Double check later if reference here causes memory leak issues
    def register_listener(self,event_name,listener_object):

        if event_name in self.listeners:
            self.listeners[event_name].append(listener_object)
        else:
            self.listeners[event_name] = [listener_object]

    def unregister_listener(self,event_name,listener_object):
        #del self.listeners[event_name][listener_object]
        pass  

    def publish_event(self,event_name,event_data):
        if self.listeners[event_name] is not None:
            for listener in self.listeners[event_name]:
                getattr(listener, "event_subscribe")(event_name,event_data)



#!/usr/bin/python
import json
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class social_search_manager():

    def __init__(self):
        pass

    def run_search(self,url):
        try:
            import ssl
        except ImportError:
            print("error: no ssl support")
        
        http = urllib3.PoolManager()

        try:
            response = http.request('GET', url)
        except ImportError:
            return ["connection failure"]
        
        parsed_json = json.loads(str(response.data))
        data_list = list(parsed_json["data"]["children"])
        return_list =[]

        for post in data_list:
            return_list.append(post["data"]["title"])
            
        return return_list

   



#!/usr/bin/python
import os
import subprocess
import logging
import traceback

class history_command_manager():

        def __init__(self):
                pass

        def get_history_list(self,history_file):

                #run_command = ['bash', '-i', '-c', 'history -r; history']
                run_command = ['bash', '-i', '-c', 'history -r; history']
                command_output = "n/a"
                err = ""
                command_detail_info = ""
                out = ""
                try:
                        p = subprocess.Popen(run_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                        out, err = p.communicate()
                        command_detail_info=out
                except:
                        logging.error(str(err)+traceback.format_exc())

        
                #logging.debug("load history command: "+command_detail_info+out+err)
                
                if err is not None and err !="":
                        command_detail_info = "n/a"

                #list_of_lines = [line.strip() for line in command_detail_info]
                list_of_lines = [line.strip() for line in command_detail_info.splitlines()]
                return list_of_lines

        """
        def get_history_list(self,history_file):

                #print(os.system('help'))
                # doesnt include current session yet
                user_history = os.path.expanduser(history_file)
                lines = open(user_history).readlines()
                list_of_lines = [line.strip() for line in lines]
                return list_of_lines

        """



#!/usr/bin/python
import urwid
from urllib2 import urlopen
from HTMLParser import HTMLParser
from simplejson import loads

from list_button import list_button

from history_command_manager import history_command_manager
from settings_manager import settings_manager
from social_search_manager import social_search_manager
from command_parse_manager import command_parse_manager
from publish_subscribe_manager import publish_subscribe_manager

from terminal_overflow_search_module import terminal_overflow_search_module
from terminal_overflow_settings_module import terminal_overflow_settings_module
from terminal_overflow_command_info_module import terminal_overflow_command_info_module



import json
import sys
import logging
import os
import subprocess
import traceback

            

class terminal_overflow_layout():

################### SETTINGS ################### 

    #define initial settings
    initial_search_results = ["none"]
    current_screen = urwid.raw_display.Screen()
    screen_cols,screen_rows = current_screen.get_cols_rows()
    current_list_height = screen_rows - 6
    current_list_width = (screen_cols/2) -10
    current_command_token_list_width = (screen_cols/2) -5

    radio_category_group = []
    text_radio_list = [u"history", u"social", u"saved"]

    radios = []
    selected_category="history"
    text_title_header = '--TERMINAL-OVERFLOW'
    text_footer_text =[
        'Press ctl+(', ('help button', 'R'), ') for settings. ',
        'Press ctl+(', ('quit button', 'X'), ') to quit.']

    initial_search_result_details = {"none":["none"]}

    #Color Scheme
    palette = [    
        ('settings_box', 'dark blue','', 'bold'),
        ('settings_text', 'dark cyan', ''),
        ('settings_edit_text','light gray', 'dark blue'),
        ('settings_edit_text_focus','yellow', 'dark blue', 'bold'),
        ('settings_edit_button_save','light blue', ''),
        ('settings_edit_button_close','light blue', '', 'bold'),
        ('settings_edit_button_focus','standout', ''),
        ('result_details_header','light blue,bold', '',),
        ('result_details_focus','standout', ''),
        ('result_details_body','', '','bold'),
        ('result_details_body_focus','', '','bold'),
        ('reversed', 'standout', ''),
        ('titlebar', 'dark cyan', ''),
        ('help button', 'dark green,bold', ''),
        ('quit button', 'dark red,bold', ''),
        ('editbx','light gray', 'dark blue'),
        ('editfc','yellow', 'dark blue', 'bold'),
        ('bright','','', 'bold'),
        ('radregular','light blue', ''),
        ('radfocus','light blue','', 'bold')]
    

################### KEY EVENTS ###################

    # Handles Unhandled MainLoop keyboard input
    def handle_input(self,key):

        #Global key combos
        if key == 'ctrl x':
            raise urwid.ExitMainLoop()
        elif key == 'ctrl r':
            self.settings_popup.toggle_open_close()
            pass
 
        #Settings popup is open
        if self.settings_popup.isOpen:
            if key == 'esc':
                self.settings_popup.toggle_open_close()
        else:
            if key == 'enter':
                pass
                #self.load_search_category(True)






    def __init__(self):


################### CREATE MANAGERS ###################

        self.settings_manager = settings_manager()
        self.history_command_manager = history_command_manager()
        self.social_search_manager = social_search_manager()
        self.command_parse_manager = command_parse_manager()

        self.publish_subscribe_manager = publish_subscribe_manager()

################### CREATE MODULES ###################

        self.terminal_modules = {}
        self.terminal_modules['search'] = terminal_overflow_search_module()
        self.terminal_modules['search'].build_initial_layout()
        self.terminal_modules['search'].begin(self.publish_subscribe_manager)

        self.terminal_modules['command_info'] = terminal_overflow_command_info_module()
        self.terminal_modules['command_info'].build_initial_layout()
        self.terminal_modules['command_info'].begin(self.publish_subscribe_manager)


################### PUBLISH/SUBSCRIBE ###################

        


################### CREATE UI ###################

        # Place Holder
        #self.main_spot_1 = urwid.WidgetPlaceholder(self.menu_original)

        # Create Header
        self.settings_popup = terminal_overflow_settings_module(self.text_title_header,self.palette,self.settings_manager)
        self.header = urwid.AttrMap(self.settings_popup, 'titlebar')


        self.w = urwid.Columns(
            [
                ('weight',1,self.terminal_modules['search'].get_base_layout_widget()),
                ('weight',1,self.terminal_modules['command_info'].get_base_layout_widget())
            ]
             ,dividechars=1, focus_column=0)

        # Assemble the widgets into the widget layout
        self.layout = urwid.Frame(header=self.header, body=self.w, footer=None)


        # Create the event loop
        self.main_loop = urwid.MainLoop(self.layout, self.palette, unhandled_input=self.handle_input,pop_ups=True)
        self.main_loop.run()


log_dir = os.path.expanduser("terminal_overflow.log")
logging.basicConfig(filename=log_dir,level=logging.DEBUG)
logging.debug('Application startup')

main_terminal = terminal_overflow_layout()








name = "terminaloverflow"
import terminal_overflow_loader

def main():
    print("TERMINALOVERFLOW")
    log_dir = os.path.expanduser("terminal_overflow.log")
    logging.basicConfig(filename=log_dir,level=logging.DEBUG)
    logging.debug('Application startup')

    main_terminal = terminal_overflow_layout()


import logging
import os
import subprocess
import traceback
import re

class command_parse_manager():

    def __init__(self):
        pass

    def build_command_detail_info_dictionary(self,command_string):
        command_list = self.parse_command_string(command_string)
        command_info_dictionary = {}

        for the_command in command_list:
            tmp_cmd_item = self.get_command_detail_info(the_command)
            if tmp_cmd_item != "n/a":
                command_info_dictionary[tmp_cmd_item] = tmp_cmd_item

        return command_info_dictionary
     
    def get_command_detail_info(self,command_name):
        
        run_command = "whatis " + command_name    
        command_detail_info = "n/a"
        err = ""
        try:
            p = subprocess.Popen(run_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE,shell=True)
            out, err = p.communicate()
            command_detail_info=out
        except:
            logging.error(str(err)+traceback.format_exc())

       
        logging.debug("command: "+command_detail_info+err)
        
        if err is not None and err !="":
            command_detail_info = "n/a"
        else:
            command_detail_info = command_detail_info.strip()
            #format the string?
            pass

        return command_detail_info

    def parse_command_string(self,command_string):
        #TODO make this more advanced
        command_tokens = list(command_string.split(" "))

        logging.debug("command tokens: " + str(command_tokens[1:]))
        return command_tokens[1:]
    

    ### parse command details (extended command info)
    def get_command_extended_info(self,command_name):
        
        run_command = "man " + command_name    
        command_detail_info = "n/a"
        err = ""
        try:
            p = subprocess.Popen(run_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE,shell=True)
            out, err = p.communicate()
            command_detail_info=out
        except:
            logging.error(str(err)+traceback.format_exc())
     
        #logging.debug("man for : "+command_detail_info+err)
        
        parsed_command_detail_string = ""

        if err is not None and err !="":
            pass
        else:
            #get just the OPTIONS or DESCRIPTION or SYNOPSIS
            pattern_start = re.compile("SYNOPSIS")
            pattern_end = re.compile("\w")
            has_matched = False

            #Get SYNOPSYS text from man
            for manline in command_detail_info.splitlines():
                if has_matched:
                    
                    if pattern_end.match(manline):
                        break
                    else:
                        parsed_command_detail_string += manline + "\n"

                elif pattern_start.match(manline):
                    has_matched = True
            reg_results_start_pos = pattern_start.match(command_detail_info)

        if parsed_command_detail_string == "":
            parsed_command_detail_string = "Command details not found"    
        return parsed_command_detail_string

   



#!/usr/bin/python


import urwid
from urllib2 import urlopen
from HTMLParser import HTMLParser
from simplejson import loads

from history_command_manager import history_command_manager
from settings_manager import settings_manager
from social_search_manager import social_search_manager
from list_button import list_button
from command_parse_manager import command_parse_manager

import json
import sys
import logging
import os
import subprocess
import traceback



class terminal_overflow_command_info_module():

################### SETTINGS ################### 

    #define initial settings
    initial_search_results = ["none"]
    current_screen = urwid.raw_display.Screen()
    screen_cols,screen_rows = current_screen.get_cols_rows()
    current_list_height = screen_rows - 6
    current_list_width = (screen_cols/2) -10
    current_command_token_list_width = (screen_cols/2) -5


    initial_search_result_details = {"none":"none"}

    palette = []


################### KEY EVENTS ###################

        # Handles Unhandled MainLoop keyboard input
    def handle_input(self,key):
        pass


################### CLICK EVENTS ###################
 
    #Command
    def command_token_chosen(self,button, choice):

        command_token_list = choice.split(" ")
        command_token = str(command_token_list[0])

        details_text = self.command_parse_manager.get_command_extended_info(command_token)
        details_text = unicode(details_text, 'utf-8')
        self.command_token_details_text.set_edit_text(details_text)

    #Search
    def item_chosen(self,button, choice):
        self.result_details.set_edit_text(choice)      
        search_result_command_details = self.command_parse_manager.build_command_detail_info_dictionary(choice)
        #Refresh command info details
        self.main_result_details_list.original_widget = self.generate_result_details(u'', search_result_command_details)  
        
        #Fill command details
        command_token_default = ""
        for command in search_result_command_details:
            command_token_default = command
        
        self.command_token_chosen(None,command_token_default)

   

################### FORMAT ###################
    #Command Info
    def format_command_info_result(self,unformatted_result):
        max_line_width = self.current_command_token_list_width    
        formatted_result = ""
        formatted_result = unformatted_result[:max_line_width]  
        return formatted_result


################### GENERATE ###################

    def generate_result_details(self, title, command_selected_dict):

        body = []
        for c in command_selected_dict:
            detail_display_text = self.format_command_info_result(command_selected_dict[c])
            button = list_button(detail_display_text)
            urwid.connect_signal(button, 'click', self.command_token_chosen, detail_display_text)
            #button = urwid.Padding(button,align='left',width='clip')
            body.append(urwid.AttrMap(button,None, focus_map='reversed'))    

        return urwid.ListBox(urwid.SimpleFocusListWalker(body))





    def __init__(self):
 

################### CREATE MANAGERS ###################
        self.settings_manager = settings_manager()
        self.command_parse_manager = command_parse_manager()



################### GET BASE LAYOUT ###################
    def get_base_layout_widget(self):
        return self.layout


################### BEGIN ###################
    def begin(self,publish_subscribe_manager):

        self.publish_subscribe_manager = publish_subscribe_manager
        self.publish_subscribe_manager.register_listener('search_input',self)

################### SUBSCRIBE EVENT ###################
    def event_subscribe(self,event_name,event_data):

        if event_name == 'search_input':
            self.item_chosen(None,event_data)


################### CREATE UI ###################
    def build_initial_layout(self):
        

        # Application Box
        self.result_details_header = urwid.AttrWrap(urwid.Text(u'Command:'),'result_details_header','result_details_focus')
        self.result_details = urwid.Edit(u'')

        # Info
        self.result_details_list_header = urwid.AttrWrap(urwid.Text(u'Info:'),'result_details_header','result_details_focus')
        self.main_result_details_list = urwid.BoxAdapter(self.generate_result_details(u'', self.initial_search_result_details),self.current_list_height/3)

        # Details pile
        self.command_token_details_header = urwid.AttrWrap(urwid.Text(u'Details:'),'result_details_header','result_details_focus')
        self.command_token_details_text = urwid.Edit(u'')
        self.pile_results = urwid.Pile([self.result_details_header, 
                                        self.result_details,
                                        self.result_details_list_header, 
                                        self.main_result_details_list,
                                        self.command_token_details_header,
                                        self.command_token_details_text
                                        ])

        self.details_filler = urwid.AttrMap(urwid.Filler(self.pile_results, valign='top', top=0, bottom=1),'result_details_body','result_details_body_focus')
        self.result_box_details = urwid.LineBox(self.details_filler)
        self.w = urwid.Columns([('weight',1,self.result_box_details)],dividechars=1, focus_column=0)
        self.layout = urwid.Frame(header=None, body=self.w, footer=None)


logging.debug('Loading Command Info Module')


